import sys

from showmereqs.analyze import get_third_party_imports
from showmereqs.generate import generate_reqs
from showmereqs.package_info import PackageInfo


def main():
    args = sys.argv
    print(args)
    if len(args) < 2:
        print("Usage: showmereqs <path_to_project>")
        return

    third_party_imports = get_third_party_imports(args[1])
    third_party_package_infos = [
        PackageInfo(import_name) for import_name in third_party_imports
    ]
    generate_reqs(third_party_package_infos)


if __name__ == "__main__":
    sys.exit(main(sys.argv))



import importlib.metadata
from typing import Optional

import requests

from showmereqs.utils import get_special_mapping


class PackageInfo:
    """a class to get package info, including package name, version, etc."""

    json_api = "https://pypi.org/pypi"

    def __init__(self, import_name: str):
        self.import_name = import_name
        self.version: Optional[str] = self._get_local_version()

        self.mapping_name: Optional[str] = self._get_package_name_from_special_mapping()

        self.package_name: Optional[str] = None
        if self.mapping_name is None:
            json_info = self._get_pypi_json(self.import_name)
            if json_info is not None:
                self.package_name = json_info["info"]["name"]
        else:
            self.package_name = self.mapping_name

    def __str__(self):
        return f"<PackageInfo> {{\nimport_name: {self.import_name}\nversion: {self.version}\npackage_name: {self.package_name}\n}}"

    def format_row(self):
        return (
            self.version is not None,
            self.package_name is not None,
            self.format_version_info(),
            self.format_import_info(),
        )

    def format_version_info(
        self,
        eq_sign: str = "==",
    ):
        if self.version is None:
            return f"{self.package_name}"
        version_txt = f"{self.package_name}{eq_sign}{self.version}"
        return f"{version_txt}"

    def format_import_info(self):
        return f"# {self.import_name}"

    def _get_local_version(self):
        try:
            version = importlib.metadata.version(self.import_name)
            return version
        except importlib.metadata.PackageNotFoundError:
            return None

    def _get_package_name_from_special_mapping(
        self, special_mapping: dict[str, str] = None
    ):
        if special_mapping is None:
            special_mapping = get_special_mapping()
        if self.import_name in special_mapping:
            return special_mapping[self.import_name]
        return None

    def _get_pypi_json(self, package_name: str):
        api = f"{self.json_api}/{package_name}/json"
        try:
            response = requests.get(api, timeout=2)
            if response.status_code == 200:
                return response.json()
            else:
                return None
        except Exception as e:
            print(f"Warning: Error checking {package_name} on PyPI: {e}")



import ast
import sys
from pathlib import Path

from .utils import get_builtin_modules, get_ignore_dirs


def traversal_directory(directory: str) -> tuple[set[str], set[str]]:
    """get all imports and local modules in a directory"""
    all_imports = set()
    local_modules = set()

    # recursive traversal directory
    for file in get_py_files(directory):
        imports = get_imports_from_file(file)
        all_imports.update(imports)

        add_local_modules(local_modules, file)

    return all_imports, local_modules


def get_py_files(directory: str):
    """yield .py files in a directory"""
    for file in Path(directory).iterdir():
        if file.is_file() and file.suffix == ".py":
            yield file
        elif file.is_dir():
            if file.name in get_ignore_dirs():
                continue
            yield from get_py_files(file)


def get_imports_from_file(file_path: Path) -> set[str]:
    """analyze imports in a single python file"""
    imports = set()

    try:
        with open(file_path, "r", encoding="utf-8") as file:
            tree = ast.parse(file.read())

        for node in ast.walk(tree):
            # deal import xxx
            if isinstance(node, ast.Import):
                for name in node.names:
                    imports.add(name.name.split(".")[0])

            # deal from xxx import yyy
            elif isinstance(node, ast.ImportFrom):
                if node.module:
                    imports.add(node.module.split(".")[0])

    except Exception as e:
        print(f"analyze {file_path} error: {str(e)}")

    return imports


def add_local_modules(local_modules: set[str], file: Path):
    """add local modules to set"""
    for p in file.parents:
        local_modules.add(p.name)
    local_modules.add(file.name.replace(".py", ""))


def filter_imports(imports: set[str], local_modules: set[str]) -> set[str]:
    """filter imports that are not in the local directory and not in the built-in modules"""
    built_in_modules = get_builtin_modules()

    return imports - local_modules - built_in_modules


def get_third_party_imports(dir: str) -> set[str]:
    """
    get all third party imports from a directory,
    exclude local modules and built-in modules
    """
    all_imports, local_modules = traversal_directory(dir)
    return filter_imports(all_imports, local_modules)


if __name__ == "__main__":
    print(get_third_party_imports("D:/Desktop/showmereqs/tests/all_kinds_import"))



"""
This module contains utility functions for ShowMeReqs.

Some package mapping and stdlib data are derived from pipreqs:
https://github.com/bndr/pipreqs/tree/master
Licensed under Apache License, Version 2.0
"""

import json
import sys
from pathlib import Path

config_dir = Path(__file__).parent / "config"
special_mapping_path = config_dir / "special_mapping"
stdlib_path = config_dir / "stdlib"
ignore_path = config_dir / "ignore.json"

special_mapping: dict[str, str] = {}
stdlib_modules: set[str] = set()
ignore_dirs: set[str] = set()


def get_special_mapping():
    global special_mapping
    if len(special_mapping) == 0:
        with open(special_mapping_path, "r") as f:
            for line in f.read().splitlines():
                import_name, package_name = line.strip().split(":")
                special_mapping[import_name] = package_name

    return special_mapping


def get_builtin_modules() -> set[str]:
    """get python builtin modules"""
    global stdlib_modules
    if len(stdlib_modules) != 0:
        return stdlib_modules

    # method after Python 3.10+
    if hasattr(sys, "stdlib_module_names"):
        return set(sys.stdlib_module_names)

    # method before Python 3.10
    else:
        with open(stdlib_path, "r") as f:
            stdlib_modules = set(f.read().splitlines())
    return stdlib_modules


def get_ignore_dirs():
    global ignore_dirs
    if len(ignore_dirs) != 0:
        return ignore_dirs

    with open(ignore_path, "r") as f:
        ignore_dirs = set(json.load(f)["ignore_dirs"])

    return ignore_dirs


if __name__ == "__main__":
    special_mapping = get_special_mapping()
    builtin_modules = get_builtin_modules()

    print(special_mapping)



import sys
from typing import TextIO

from showmereqs.package_info import PackageInfo


def write_block(
    header: str,
    max_version_len: int,
    infos: list[tuple[str, str]],
    f: TextIO,
    version_prefix: str = "",
):
    if len(infos) == 0:
        return

    f.write(f"# {header}\n")
    _txt = "[package]"
    f.write(f"# {_txt: <{max_version_len+1}}[import name]\n")

    for info in infos:
        version_txt = info[0]
        f.write(f"{version_prefix}{version_txt: <{max_version_len+1}}")
        f.write(info[1] + "\n")

    f.write("\n")


def sort_row_txts(row_txts: list[tuple[bool, bool, str, str]]):
    full_info = [(x[2], x[3]) for x in row_txts if x[0] and x[1]]
    no_version = [(x[2], x[3]) for x in row_txts if not x[0] and x[1]]
    no_package_name = [(x[2], x[3]) for x in row_txts if x[0] and not x[1]]
    no_info = [(x[2], x[3]) for x in row_txts if not x[0] and not x[1]]

    full_info.sort()
    no_version.sort()
    no_package_name.sort()
    no_info.sort()
    return full_info, no_version, no_package_name, no_info


def generate_reqs(package_infos: list[PackageInfo]):
    max_package_len = 0
    row_txts: list[tuple[bool, bool, str, str]] = []
    for package_info in package_infos:
        max_package_len = max(
            max_package_len,
            len(package_info.format_version_info()),
        )
        row_txts.append(package_info.format_row())

    full_info, no_version, no_package_name, no_info = sort_row_txts(row_txts)

    with open("requirements.txt", "w") as f:
        f.write("# generated by showmereqs\n")
        f.write(f"# python version: {sys.version}\n\n")

        write_block("package with version", max_package_len, full_info, f)
        write_block("package without version", max_package_len, no_version, f)
        write_block(
            "package without package name, please check to find the correct package name",
            max_package_len,
            no_package_name,
            f,
            "# ",
        )
        write_block(
            "package without version and package name",
            max_package_len,
            no_info,
            f,
            "# ",
        )
